#@IgnoreInspection BashAddShebang
# Base app functionality (Koalephant Shell Script Library)
# Version: PACKAGE_VERSION
# Copyright: 2014, Koalephant Co., Ltd
# Author: Stephen Reay <stephen@koalephant.com>, Koalephant Packaging Team <packages@koalephant.com>

# Base Shell Library name
readonly KOALEPHANT_LIB_NAME="PACKAGE_VENDOR-PACKAGE_NAME"

# Base Shell library owner
readonly KOALEPHANT_LIB_OWNER="Koalephant Co., Ltd"

# Base Shell Library version
readonly KOALEPHANT_LIB_VERSION="PACKAGE_VERSION"

# Base Shell Library year
readonly KOALEPHANT_LIB_YEAR="2014-2019"

# Base directory for the installed Library
readonly KOALEPHANT_LIB_PATH="${KOALEPHANT_LIB_PATH:-PREFIX/LIBRARY_PATH}"

# Get the tool name
#
# Input:
# $KOALEPHANT_TOOL_NAME - explicit name used as-is if set
# $0 - the script name is stripped of directory components and any '.sh' suffix
#
# Output:
# the tool name
k_tool_name() {
	if [ -n "${KOALEPHANT_TOOL_NAME:-}" ]; then
		printf -- '%s\n' "${KOALEPHANT_TOOL_NAME}"
	else
		k_fs_basename "${0}" ".sh"
	fi
}

# Get the tool version
#
# Input:
# $KOALEPHANT_TOOL_VERSION - explicit version used as-is if set. Default is 1.0.0 if not set
#
# Output:
# the tool version
k_tool_version() {
	printf -- '%s\n' "${KOALEPHANT_TOOL_VERSION:-1.0.0}"
}

# Get the tool copyright years
#
# Input:
# $KOALEPHANT_TOOL_YEAR - explicit copyright year(s) used as-is if set. Default is current year if not set
#
# Output:
# the tool copyright year(s)
k_tool_year() {
	printf -- '%s\n' "${KOALEPHANT_TOOL_YEAR:-$(date +%Y)}"
}

# Get the tool copyright owner
#
# Input:
# $KOALEPHANT_TOOL_OWNER - explicit copyright owner used as-is if set. Default is 'Koalephant Co., Ltd' if not set
#
# Output:
# the tool copyright owner
k_tool_owner() {
	printf -- '%s\n' "${KOALEPHANT_TOOL_OWNER:-${KOALEPHANT_LIB_OWNER}}"
}

# Get the tool description
#
# Input:
# $KOALEPHANT_TOOL_DESCRIPTION - tool description used as-is
#
# Output:
# the tool description
k_tool_description() {
	printf -- '%s\n' "${KOALEPHANT_TOOL_DESCRIPTION:-}"
}

# Get the tool options descriptions
#
# Input:
# $KOALEPHANT_TOOL_OPTIONS - tool options descriptions used as-is
#
# Output:
# the tool options descriptions
k_tool_options() {
	printf -- '%s\n' "${KOALEPHANT_TOOL_OPTIONS:-}"
}

# Get the tool option(s) argument(s)
# Shows output if (#k_tool_options) returns non-empty content
#
# Input:
# $KOALEPHANT_TOOL_OPTIONS_ARGUMENTS - explicit tool option(s) argument(s) used as-is. Default is '[options]' if not set
#
# Output:
# the tool option(s) argument(s)
k_tool_options_arguments() {
	if [ -n "$(k_tool_options)" ]; then
		printf -- ' %s\n' "${KOALEPHANT_TOOL_OPTIONS_ARGUMENTS:-[options]}"
	fi
}

# Get the tool arguments
#
# Input:
# $KOALEPHANT_TOOL_ARGUMENTS - tool arguments used as-is
#
# Output:
# the tool arguments
k_tool_arguments() {
	printf -- ' %s\n' "${KOALEPHANT_TOOL_ARGUMENTS:-}"
}

# Get the tool enviromnment variables
#
# Input:
# $KOALEPHANT_TOOL_ENVIRONMENT - tool environment used as-is
#
# Output
# the tool environmnet variables
k_tool_environment() {
	printf -- 'Environment: \n\n%s\n' "$(printf -- '%s' "${KOALEPHANT_TOOL_ENVIRONMENT:-None}" | sed 'G')"
}

# Get the tool usage
# uses (#k_tool_name), (#k_tool_options_arguments) and (#k_tool_arguments)
#
# Output:
# the tool usage
k_tool_usage() {
	printf -- 'Usage: %s%s%s\n' "$(k_tool_name)" "$(k_tool_options_arguments)" "$(k_tool_arguments)"
}

# Show the current tool version
# uses (#k_tool_name), (#k_tool_version) (#k_tool_year) and (#k_tool_owner)
#
# Output:
# the version information
k_version() {
	printf -- '%s version %s \nCopyright (c) %s, %s\n' "$(k_tool_name)" "$(k_tool_version)" "$(k_tool_year)" "$(k_tool_owner)"
}

# Show the current library version
#
# Input:
# $KOALEPHANT_LIB_NAME
# $KOALEPHANT_LIB_VERSION
# $KOALEPHANT_LIB_YEAR
# $KOALEPHANT_LIB_OWNER
#
# Output:
# the version information
k_library_version () {
	printf -- '%s version %s \nCopyright (c) %s, %s\n' "${KOALEPHANT_LIB_NAME}" "${KOALEPHANT_LIB_VERSION}" "${KOALEPHANT_LIB_YEAR}" "${KOALEPHANT_LIB_OWNER}"
}

# Show either the full or minimal tool version
# uses (#k_tool_version) and (#k_version)
#
# Input:
# $0 - flag indicating whether version info should be shown.
# $1 - flag indicating whether just the version number should be shown
# $2 - flag indicating whether the Library info/version should be shown as well
#
# Output:
# the version information/number
#
# Return:
# 0 if version info is shown, 1 otherwise
k_version_helper () {
	# shellcheck disable=SC2039
	local showVersion quiet verbose

	k_requires_args k_version_helper $# 3

	showVersion="${1}"
	quiet="${2}"
	verbose="${3}"

	if [ "${showVersion}" = true ]; then
		if [ "${quiet}" = true ]; then
			k_tool_version
			if [ "${verbose}" = true ]; then
				printf -- '%s\n' "${KOALEPHANT_LIB_VERSION}"
			fi
		else
			k_version
			if [ "${verbose}" = true ]; then
				k_library_version
			fi
		fi
		return 0
	fi

	return 1
}

# Show the Usage info for this script
# uses (#k_tool_usage), (#k_tool_description) and (#k_tool_options) and (#k_tool_environment)
#
# Output:
# the usage information
k_usage() {
	printf -- '%s\n%s\n\n%s\n\n%s\n' "$(k_tool_usage)" "$(k_tool_description)" "$(k_tool_options)" "$(k_tool_environment)"
}

# Check if the running (effective) user is root, and error if not
#
# Input:
# $1 - an extra error message to show if the current user is not root
#
# Output:
# Any additional message provided in `$1` plus a standard error message, if run by a non-root user.
#
# Return:
# 0 when effective user is root, 1 otherwise
k_require_root() {
	# shellcheck disable=SC2039
	local message="${1:-}"
	if [ "$(id -u)" -ne 0 ]; then
		if [ -n "${message}" ]; then
			k_log_err "${message}"
		fi
		k_log_err '%s must be run as root' "$(k_tool_name)"
		return 1
	fi
}

# Check that the function has n arguments specified
#
# Input:
# $1 - the name of the function
# $2 - the number of arguments provided
# $3 - the number of arguments required. Defaults to 1 if not specified
#
# Output:
# Error message written to stderr when applicable
#
# Return:
# 0 when required number of args passed, 1 otherwise
k_requires_args() {
	# shellcheck disable=SC2039
	local name provided req
	if [ $# -eq 2 ] || [ $# -eq 3 ]; then
		name="$1"
		provided="$2"
		if [ $# -eq 3 ]; then
			req="$3"
		else
			req=1
		fi
	else
		name=k_requires_args
		provided=$#
		req=2
	fi

	if [ "${provided}" -lt "${req}" ]; then
		printf -- '%s requires at least %d arguments, %d given\n' "${name}" "${req}" "${provided}" >&2
		exit 1
	fi
}

# Check that an option has an argument specified.
#
# Input:
# $1 - the option name
# $2 - the next argument the tool was invoked with
#
# Output:
# the argument value if valid
#
# Return:
# 0 on success, 1 otherwise
#
# Deprecated:
# As of v2.5.0, use (#k_options_split) and (#k_options_arg_required) instead
k_option_requires_arg() {
	# shellcheck disable=SC2039
	local name
	k_requires_args k_option_requires_arg $#
	name="$1"

	k_log_deprecated k_option_requires_arg 2.5.0 'k_options_split() and k_options_arg_required()'

	if [ -n "${2:-}" ] && ! k_string_starts_with "-" "${2}"; then
		printf -- '%s\n' "${2}"
	else
		k_log_err 'Error, %s requires an argument' "${name}"
		k_usage
		return 1
	fi
}

# Check whether an option has an optional argument specified.
#
# Input:
# $1 - the option name
# $2 - the next argument the tool was invoked with
#
# Output:
# the argument value if specified
#
# Deprecated:
# As of v2.5.0, use (#k_options_split) and (#k_options_arg_optional) instead
k_option_optional_arg() {
	k_log_deprecated k_option_optional_arg 2.5.0 'k_options_split() and k_options_arg_optional()'

	if [ -n "${2:-}" ] && ! k_string_starts_with "-" "${2}"; then
		printf -- '%s\n' "${2}"
	fi
}

# Read options passed and split option/value pairs.
#
# Sets three variables to be used within the context of a `while` loop:
# $OPTION_NAME - the name of the current option
# $OPTION_VALUE - the (potentially) value of the current option
# $OPTION_ARG_COUNT - a value indicating how many values should be shifted when a value has been read
#
# Input:
# $1...n - The options to parse and split
#
# Output:
# the options in a space separated string
#
# Example:
# read_options() {
# 	local originalCount=$#
#
# 	while k_options_split "$@"; do
# 		case "$OPTION_NAME" in
#
# 			(-a|--arg)
# 				k_options_arg_required 'You must supply "%s"'
# 				printf -- 'Opt with value "%s"\n' "${OPTION_VALUE}" >&2
# 				shift ${OPTION_ARG_COUNT}
# 			;;
#
# 			(-b|--bool)
# 				if k_options_arg_optional; then
# 					printf -- 'Opt with optional value specified "%s"\n' "${OPTION_VALUE}" >&2
# 					shift ${OPTION_ARG_COUNT}
# 				else
# 					printf -- 'Opt without value specified\n' >&2
# 					shift
# 				fi
# 			;;
#
# 			(--)
# 				shift
# 				break
# 			;;
#
# 			(*)
# 				break
# 			;;
# 		esac
# 	done
#
#
# 	return $(( $originalCount - $# ))
# }
#
# read_options "$@" || shift $?
k_options_split() {
	# shellcheck disable=SC2039
	local rawOption="${1:-}"
	OPTION_NAME="${rawOption}"
	OPTION_VALUE="${2:-}"
	# shellcheck disable=SC2034
	OPTION_ARG_COUNT=2

	if [ $# -lt 1 ]; then
		return 1
	fi

	case "${OPTION_NAME}" in
		(*=*)
			OPTION_NAME="${rawOption%%=*}"
			OPTION_VALUE="${rawOption#${OPTION_NAME}=}"
			OPTION_ARG_COUNT=1
		;;

		(-[a-zA-Z0-9]?*) # Cannot use [[:alnum:]] in posh or mksh
			OPTION_VALUE="${rawOption#-?}"
			OPTION_NAME="${rawOption%%${OPTION_VALUE}}"
			# shellcheck disable=SC2034
			OPTION_ARG_COUNT=1
		;;
	esac
}

# Check that an option has an argument specified.
# This function is designed to work with (#k_options_split)
#
# Input:
# $1 - optional printf template to use for the error message. Defaults to 'Error, %s requires an argument', and receives the option name as argument.
# $OPTION_NAME - the option name
# $OPTION_VALUE - the next argument the tool was invoked with
#
# Return:
# 0 if the option is specified, 1 otherwise
k_options_arg_required() {
	# shellcheck disable=SC2039
	local template='Error, %s requires an argument'
	if [ -z "${OPTION_NAME:-}" ]; then
		k_log_err 'Error, no option name set. Must be used with k_options_split()'
		return 2
	fi

	if [ -n "${1:-}" ]; then
		template="$1"
	fi

	if ! k_options_arg_optional; then
		k_log_err "${template}" "${OPTION_NAME}"
		k_usage
		return 1
	fi
}

# Check whether an option has an optional argument specified.
# This function is designed to work with (#k_options_split)
#
# Input:
# $OPTION_NAME - the option name
# $OPTION_VALUE - the next argument the tool was invoked with
#
# Return:
# 0 if the option is specified, 1 otherwise
k_options_arg_optional() {
	if [ -z "${OPTION_VALUE:-}" ] || k_string_starts_with "-" "${OPTION_VALUE}"; then
		return 1
	fi
}


# Log level EMERG - system is unusable
KOALEPHANT_LOG_LEVEL_EMERG=1
readonly KOALEPHANT_LOG_LEVEL_EMERG

# Log level ALERT - action must be taken immediately
KOALEPHANT_LOG_LEVEL_ALERT=2
readonly KOALEPHANT_LOG_LEVEL_ALERT

# Log level CRIT - critical conditions
KOALEPHANT_LOG_LEVEL_CRIT=3
readonly KOALEPHANT_LOG_LEVEL_CRIT

# Log level ERR - error conditions
KOALEPHANT_LOG_LEVEL_ERR=4
readonly KOALEPHANT_LOG_LEVEL_ERR

# Log level WARNING - warning conditions
KOALEPHANT_LOG_LEVEL_WARNING=5
readonly KOALEPHANT_LOG_LEVEL_WARNING

# Log level NOTICE - normal, but significant, condition
KOALEPHANT_LOG_LEVEL_NOTICE=6
readonly KOALEPHANT_LOG_LEVEL_NOTICE

# Log level INFO - informational message
KOALEPHANT_LOG_LEVEL_INFO=7
readonly KOALEPHANT_LOG_LEVEL_INFO

# Log level DEBUG - debug-level message
KOALEPHANT_LOG_LEVEL_DEBUG=8
readonly KOALEPHANT_LOG_LEVEL_DEBUG

# Parse a log level, check it's validity and output the integer value
#
# Input:
# $1 - log level value to parse
#
# Output:
# the parsed log level as an integer
#
# Return:
# 0 if the input can be parsed into a valid log level, 1 if not
k_log_level_parse() {
	# shellcheck disable=SC2039
	local level

	k_requires_args k_log_level_parse $#

	if level="$(k_int_parse_error "${1}" 2>/dev/null)" && [ "${level}" -ge ${KOALEPHANT_LOG_LEVEL_EMERG} ] && [ "${level}" -le ${KOALEPHANT_LOG_LEVEL_DEBUG} ]; then
		printf -- '%d' "${level}"
		return 0
	fi

	printf -- 'Invalid log level specified: "%s"\n' "$1" >&2
	return 1
}

# Check if a value is a valid log level
#
# Input:
# $1 - log level value to check
#
# Return:
# 0 if the input can be parsed into a valid log level, 1 if not
k_log_level_valid() {
	k_log_level_parse "$@" > /dev/null 2>&1
}

# Get/Set the current log level
#
# Input:
# $1 - if provided, set the active log level to this value
# $KOALEPHANT_LOG_LEVEL_ACTIVE - used as explicit log level if set. Default is ($#KOALEPHANT_LOG_LEVEL_ERR) if not set
#
# Output:
# the active log level
#
# Return:
# 0 if the log level is valid, 1 if not
# shellcheck disable=SC2120
k_log_level() {
	# shellcheck disable=SC2039
	local level retCode=0

	if [ $# -gt 0 ]; then
		if ! level="$(k_log_level_parse "$1")"; then
			retCode=1
		else
			KOALEPHANT_LOG_LEVEL_ACTIVE="$level"
		fi
	fi

	printf -- '%d\n' "${KOALEPHANT_LOG_LEVEL_ACTIVE:-${KOALEPHANT_LOG_LEVEL_ERR}}"

	return ${retCode}
}

# Get the log level name from a log level value
k_log_level_name() {
	# shellcheck disable=SC2039
	local level

	k_requires_args k_log_level_name $#

	level="$(k_log_level_parse "${1}")" || return $?

	case "${level}" in
		(${KOALEPHANT_LOG_LEVEL_EMERG})
			printf -- '%s\n' "emerg"
		;;

		(${KOALEPHANT_LOG_LEVEL_ALERT})
			printf -- '%s\n' "alert"
		;;

		(${KOALEPHANT_LOG_LEVEL_CRIT})
			printf -- '%s\n' "crit"
		;;

		(${KOALEPHANT_LOG_LEVEL_ERR})
			printf -- '%s\n' "err"
		;;

		(${KOALEPHANT_LOG_LEVEL_WARNING})
			printf -- '%s\n' "warning"
		;;

		(${KOALEPHANT_LOG_LEVEL_NOTICE})
			printf -- '%s\n' "notice"
		;;

		(${KOALEPHANT_LOG_LEVEL_INFO})
			printf -- '%s\n' "info"
		;;

		(${KOALEPHANT_LOG_LEVEL_DEBUG})
			printf -- '%s\n' "debug"
		;;
	esac
}

# Check if logs should be set to syslog
#
# Input:
# $KOALEPHANT_LOG_SYSLOG - used as flag to enable syslog
#
# Return:
# 0 if syslog should be used, 1 otherwise
k_log_syslog() {
	[ "${KOALEPHANT_LOG_SYSLOG:-false}" = true ]
}

# Log a message according to a syslog level
#
# Input:
# $1 - the syslog level constant to use. One of the `KOALEPHANT_LOG_LEVEL_*` constants
# $2 - an optional printf template to use
# $3...n - the message to write
k_log_message() {
	# shellcheck disable=SC2039
	local level template='%s\n'
	k_requires_args k_log_message $# 2
	level="$(k_log_level_parse "${1}")" || return $?
	shift

	if [ $# -gt 1 ]; then
		template="${1}\n"
		shift
	fi


	# shellcheck disable=SC2119
	if [ "$(k_log_level)" -ge "${level}" ]; then
		if k_log_syslog; then
			# shellcheck disable=SC2059
			printf -- "${template}" "$@" | tee /dev/stderr | logger -t "$(k_tool_name)" -p "user.$(k_log_level_name "${level}")"
		else
			# shellcheck disable=SC2059
			printf -- "${template}" "$@" >&2
		fi
	fi
}

# Log a message at syslog-level EMERG
#
# Input:
# $1 - an optional printf template to use
# $2...n - the message arguments/string
k_log_emerg() {
	k_log_message ${KOALEPHANT_LOG_LEVEL_EMERG} "$@"
}

# Log a message at syslog-level ALERT
#
# Input:
# $1 - an optional printf template to use
# $2...n - the message arguments/string
k_log_alert() {
	k_log_message ${KOALEPHANT_LOG_LEVEL_ALERT} "$@"
}

# Log a message at syslog-level CRIT
#
# Input:
# $1 - an optional printf template to use
# $2...n - the message arguments/string
k_log_crit() {
	k_log_message ${KOALEPHANT_LOG_LEVEL_CRIT} "$@"
}

# Log a message at syslog-level ERR
#
# Input:
# $1 - an optional printf template to use
# $2...n - the message arguments/string
k_log_err() {
	k_log_message ${KOALEPHANT_LOG_LEVEL_ERR} "$@"
}

# Log a message at syslog-level WARNING
#
# Input:
# $1 - an optional printf template to use
# $2...n - the message arguments/string
k_log_warning() {
	k_log_message ${KOALEPHANT_LOG_LEVEL_WARNING} "$@"
}

# Log a message at syslog-level NOTICE
#
# Input:
# $1 - an optional printf template to use
# $2...n - the message arguments/string
k_log_notice() {
	k_log_message ${KOALEPHANT_LOG_LEVEL_NOTICE} "$@"
}

# Log a message at syslog-level INFO
#
# Input:
# $1 - an optional printf template to use
# $2...n - the message arguments/string
k_log_info() {
	k_log_message ${KOALEPHANT_LOG_LEVEL_INFO} "$@"
}

# Log a message at syslog-level DEBUG
#
# Input:
# $1 - an optional printf template to use
# $2...n - the message arguments/string
k_log_debug() {
	k_log_message ${KOALEPHANT_LOG_LEVEL_DEBUG} "$@"
}

# Log a message indicating a function is deprecated
# Internally this logs as a DEBUG message
#
# Input:
# $1 - the function name that is deprecated
# $2 - the version the function was deprecated
# $3 - suggested replacement functions to include in the message. Defaults to empty
k_log_deprecated() {
	# shellcheck disable=SC2039
	local templateNoSugg='%s is deprecated as of v%s' templateSugg='%s is deprecated as of v%s, consider using %s instead'

	k_requires_args k_log_deprecated 2

	if [ $# -eq 2 ]; then
		k_log_debug "${templateNoSugg}" "$@"
	elif [ $# -eq 3 ]; then
		k_log_debug "${templateSugg}" "$@"
	fi
}

# Compare two version strings
#
# Version suffixes (e.g. 'alpha', 'rc1' etc) should use a tilde (~), e.g.: '1.2.0~alpha'
# Valid operators are eq/= (equal), neq/!= (not equal), gt/> (greater than), lt/<  (less than), gte/>= (greater or equal), lte/<= (less or equal)
#
# Input:
# $1 - the first version to compare
# $2 - the second version to compare
# $3 - the operator to use, defaults to 'gte'.
#
# Return:
# 0 if the version comparison passes, 1 if not
k_version_compare() {
	# shellcheck disable=SC2039
	local ver1 ver2 op

	k_requires_args k_version_compare $# 2

	ver1="$1"
	ver2="$2"
	op="${3:-gte}"

	log_action() {
		k_log_debug 'Comparing if "%s" is %s "%s".' "$@"
	}

	is_greater() {
		[ "$(printf -- '%s\n%s' "${1}" "${2}" | LC_ALL=C sort -Vr | head -n 1)" = "${1}" ]
	}

	is_less() {
		[ "$(printf -- '%s\n%s' "${1}" "${2}" | LC_ALL=C sort -V | head -n 1)" = "${1}" ]
	}

	is_equal() {
		[ "$(printf -- '%s\n%s' "${1}" "${2}" | LC_ALL=C sort -Vu | wc -l)" -eq 1 ]
	}

	case "${op}" in
		(gte|'>=')
			log_action "$1" "greater than or equal to" "$2"
			if is_equal "${ver1}" "${ver2}" || is_greater "${ver1}" "${ver2}"; then
				return 0
			fi

			return 1
		;;

		(lte|'<=')
			log_action "$1" "less than or equal to" "$2"
			if is_equal "${ver1}" "${ver2}" || is_less "${ver1}" "${ver2}"; then
				return 0
			fi

			return 1
		;;

		(neq|'!=')
			log_action "$1" "not equal to" "$2"
			if is_equal "${ver1}" "${ver2}"; then
				return 1
			fi

			return 0
		;;

		(eq|'=')
			log_action "$1" "equal to" "$2"
			if is_equal "${ver1}" "${ver2}"; then
				return 0
			fi

			return 1
		;;

		(gt|'>')
			log_action "$1" "greater than" "$2"
			if ! is_equal "${ver1}" "${ver2}" && is_greater "${ver1}" "${ver2}" ; then
				return 0;
			fi

			return 1
		;;

		(lt|'<')
			log_action "$1" "less than" "$2"
			if ! is_equal "${ver1}" "${ver2}" &&  is_less "${ver1}" "${ver2}" ; then
				return 0;
			fi

			return 1
		;;

		(*)
			k_log_err 'Unrecognised operator: %s' "${op}"
			return 2
		;;
	esac

}
