#!/bin/sh -eu

. ../base.lib.sh
. ../fs.lib.sh
. ../string.lib.sh
. ../bool.lib.sh
. ../number.lib.sh

test_k_library_version (){
	assertEquals "Lib name is correct" "$(printf -- "${KOALEPHANT_LIB_NAME} version ${KOALEPHANT_LIB_VERSION} \nCopyright (c) ${KOALEPHANT_LIB_YEAR}, ${KOALEPHANT_LIB_OWNER}")" "$(k_library_version)"
}

test_k_tool_name() {
	assertEquals "Tool name auto detect is correct" "$(basename "$0")" "$(k_tool_name)"
	assertEquals "Tool name variable is correct" "sometoolname" "$(KOALEPHANT_TOOL_NAME=sometoolname k_tool_name)"
}

test_k_tool_version() {
	assertEquals "Tool version defaults to 1.0.0" "1.0.0" "$(k_tool_version)"
	assertEquals "Tool version variable is correct" "2.3.1" "$(KOALEPHANT_TOOL_VERSION=2.3.1 k_tool_version)"
}

test_k_tool_year (){
	assertEquals "Tool year auto detect is correct" "$(date +%Y)" "$(k_tool_year)"
	assertEquals "Tool year variable is correct" "1992" "$(KOALEPHANT_TOOL_YEAR=1992 k_tool_year)"
}

test_k_tool_owner (){
	assertEquals "Tool owner default is correct" "Koalephant Co., Ltd" "$(k_tool_owner)"
	assertEquals "Tool owner variable is correct" "sometoolowner" "$(KOALEPHANT_TOOL_OWNER=sometoolowner k_tool_owner)"
}

test_k_tool_description (){
	assertEquals "Tool description variable is correct" "description foo" "$(KOALEPHANT_TOOL_DESCRIPTION="description foo" k_tool_description)"
}

test_k_tool_options (){
	assertEquals "Tool options variable is correct" "options foo" "$(KOALEPHANT_TOOL_OPTIONS="options foo" k_tool_options)"
}

test_k_tool_options_arguments (){
	assertEquals "Tool options arguments default is correct" " [options]" "$(KOALEPHANT_TOOL_OPTIONS=options k_tool_options_arguments)"
	assertEquals "Tool options arguments variable is correct" " args" "$(KOALEPHANT_TOOL_OPTIONS=options KOALEPHANT_TOOL_OPTIONS_ARGUMENTS="args" k_tool_options_arguments)"
}

test_k_tool_arguments (){
	assertEquals "Tool arguments variable is correct" " args" "$(KOALEPHANT_TOOL_ARGUMENTS=args k_tool_arguments)"
}

test_k_tool_environment (){
	assertEquals "Environment default is correct" "$(printf -- 'Environment: \n\n%s\n' 'None')" "$(k_tool_environment)"
	assertEquals "Environment variable is correct" "$(printf -- 'Environment: \n\n%s\n' 'TEST - test argument name')" "$(KOALEPHANT_TOOL_ENVIRONMENT="TEST - test argument name" k_tool_environment)"
}


test_k_version (){
	assertEquals "Version default is correct" "$(printf -- "%s version %s \nCopyright (c) %s, %s" "$(basename "$0")" "1.0.0" "$(date +%Y)" "Koalephant Co., Ltd")" "$(k_version)"
	assertEquals "Version variable is correct" "$(printf -- "%s version %s \nCopyright (c) %s, %s" "sometoolname" "2.3.1" "1992" "sometoolowner")" "$(KOALEPHANT_TOOL_NAME=sometoolname KOALEPHANT_TOOL_VERSION=2.3.1 KOALEPHANT_TOOL_YEAR=1992 KOALEPHANT_TOOL_OWNER=sometoolowner k_version)"
}

test_k_version_compare (){

	k_version_compare "1.0.0" "1.0.0" eq
	assertTrue "1.0.0 = 1.0.0" $?

	k_version_compare "1.0.0" "2.0.0" eq
	assertFalse "1.0.0 = 2.0.0" $?

	k_version_compare "1.0.0" "1.0.0" neq
	assertFalse "1.0.0 != 1.0.0" $?

	k_version_compare "1.0.0" "2.0.0" neq
	assertTrue "1.0.0 != 2.0.0" $?

	k_version_compare "1.0.0" "1.0.0" gte
	assertTrue "1.0.0 >= 1.0.0" $?

	k_version_compare "1.0.1" "1.0.0" gte
	assertTrue "1.0.1 >= 1.0.0" $?

	k_version_compare "1.0.1" "2.0.0" gte
	assertFalse "1.0.1 >= 2.0.0" $?

	k_version_compare "1.0.1" "1.0.0" lte
	assertFalse "1.0.1 <= 1.0.0" $?

	k_version_compare "0.0.1" "1.0.0" lte
	assertTrue "0.0.1 <= 1.0.0" $?

	k_version_compare "1.0.1" "1.0.2" lt
	assertTrue "1.0.1 < 1.0.2" $?

	k_version_compare "1.0.0" "1.0.0" gt
	assertFalse "1.0.0 > 1.0.0" $?

	k_version_compare "1.0.0" "0.9.9" gt
	assertTrue "1.0.0 > 0.9.9" $?

	k_version_compare "1.0.0" "1.0.0~alpha" gt
	assertTrue "1.0.0 > 1.0.0~alpha" $?
}

worker_k_log_levels() {
	local setLevel output='' logFunc="$1" msgLevel="$2" msgLevelName setLevelName allLevels="${KOALEPHANT_LOG_LEVEL_EMERG} ${KOALEPHANT_LOG_LEVEL_ALERT} ${KOALEPHANT_LOG_LEVEL_CRIT} ${KOALEPHANT_LOG_LEVEL_ERR} ${KOALEPHANT_LOG_LEVEL_WARNING} ${KOALEPHANT_LOG_LEVEL_NOTICE} ${KOALEPHANT_LOG_LEVEL_INFO} ${KOALEPHANT_LOG_LEVEL_DEBUG}"

	msgLevelName="$(k_log_level_name "${msgLevel}")"

	for setLevel in ${allLevels}; do
		KOALEPHANT_LOG_LEVEL_ACTIVE=${setLevel}
		setLevelName="$(k_log_level_name ${setLevel})"
		output="$(${logFunc} "foo bar" 2>&1 1>/dev/null)"
		assertTrue 'Message is logged without error' $?
		if [ ${msgLevel} -le ${setLevel} ]; then
			assertEquals "${msgLevelName} is logged at level ${setLevelName}" "foo bar" "${output}"
		else
			assertEquals "${msgLevelName} is not logged at level ${setLevelName}" "" "${output}"
		fi
	done
}

test_k_log_emerg (){
	worker_k_log_levels k_log_emerg ${KOALEPHANT_LOG_LEVEL_EMERG}
}

test_k_log_alert (){
	worker_k_log_levels k_log_alert ${KOALEPHANT_LOG_LEVEL_ALERT}
}


test_k_log_crit (){
	worker_k_log_levels k_log_crit ${KOALEPHANT_LOG_LEVEL_CRIT}
}


test_k_log_err (){
	worker_k_log_levels k_log_err ${KOALEPHANT_LOG_LEVEL_ERR}
}


test_k_log_warning (){
	worker_k_log_levels k_log_warning ${KOALEPHANT_LOG_LEVEL_WARNING}
}


test_k_log_notice (){
	worker_k_log_levels k_log_notice ${KOALEPHANT_LOG_LEVEL_NOTICE}
}


test_k_log_info (){
	worker_k_log_levels k_log_info ${KOALEPHANT_LOG_LEVEL_INFO}
}


test_k_log_debug (){
	worker_k_log_levels k_log_debug ${KOALEPHANT_LOG_LEVEL_DEBUG}
}


test_k_log_deprecated (){
	KOALEPHANT_LOG_SYSLOG=false
	KOALEPHANT_LOG_LEVEL_ACTIVE=${KOALEPHANT_LOG_LEVEL_DEBUG}

	output="$(k_log_deprecated foo_func 2.0 2>&1 1>/dev/null)"
	assertEquals 'Output is standard error message with 2 arguments' 'foo_func is deprecated as of v2.0' "${output}"

	output="$(k_log_deprecated foo_func 2.0 'foobar_func' 2>&1 1>/dev/null)"
	assertEquals 'Output is extended error message with 3 arguments' 'foo_func is deprecated as of v2.0, consider using foobar_func instead' "${output}"
}

test_k_log_message (){
	local output msgLevel allLevels="${KOALEPHANT_LOG_LEVEL_EMERG} ${KOALEPHANT_LOG_LEVEL_ALERT} ${KOALEPHANT_LOG_LEVEL_CRIT} ${KOALEPHANT_LOG_LEVEL_ERR} ${KOALEPHANT_LOG_LEVEL_WARNING} ${KOALEPHANT_LOG_LEVEL_NOTICE} ${KOALEPHANT_LOG_LEVEL_INFO} ${KOALEPHANT_LOG_LEVEL_DEBUG}"

	for msgLevel in ${allLevels}; do
		worker_k_log_levels "k_log_message ${msgLevel}" "${msgLevel}"
	done

	output="$(k_log_message ${KOALEPHANT_LOG_LEVEL_EMERG} 'foo %s' bar  2>&1 1>/dev/null)"
	assertEquals 'Custom format log message is rendered correctly' 'foo bar' "${output}"

	output="$(k_log_message ${KOALEPHANT_LOG_LEVEL_EMERG} 'foo bar baz' 2>&1 1>/dev/null)"
	assertEquals 'Default format log message is rendered correctly' 'foo bar baz' "${output}"

	k_log_message 9 foo 2>/dev/null
	assertFalse 'Invalid log level returns error' $?

	k_log_message 0 foo 2>/dev/null
	assertFalse 'Invalid log level returns error' $?

	k_log_message warn foo 2>/dev/null
	assertFalse 'Invalid log level returns error' $?
}

test_k_log_syslog (){
	unset KOALEPHANT_LOG_SYSLOG

	k_log_syslog
	assertFalse "Syslog defaults to off" $?

	KOALEPHANT_LOG_SYSLOG=true k_log_syslog
	assertTrue "Syslog explicitly on" $?

	KOALEPHANT_LOG_SYSLOG=false k_log_syslog
	assertFalse "Syslog explicitly off" $?
}

test_k_log_level (){
	unset KOALEPHANT_LOG_LEVEL_ACTIVE
	assertEquals "Default log level is Error" "${KOALEPHANT_LOG_LEVEL_ERR}" "$(k_log_level)"

	KOALEPHANT_LOG_LEVEL_ACTIVE=${KOALEPHANT_LOG_LEVEL_DEBUG}
	assertEquals "Previously set log level is read" "${KOALEPHANT_LOG_LEVEL_DEBUG}" "$(k_log_level)"

	result1="$(k_log_level ${KOALEPHANT_LOG_LEVEL_NOTICE})"
	assertTrue "Setting valid log level succeeds" $?
	assertEquals "Setting log level outputs same on success" "${KOALEPHANT_LOG_LEVEL_NOTICE}" "${result1}"

	before2="$(k_log_level)"
	result2="$(k_log_level 9 2>/dev/null)"
	assertFalse "Setting invalid log level fails" $?
	assertEquals "Setting invalid log level outputs current level" "${before2}" "${result2}"
}



test_k_log_level_name (){
	assertEquals "Log level name is Emerg" "emerg" "$(k_log_level_name ${KOALEPHANT_LOG_LEVEL_EMERG})"
	assertEquals "Log level name is Alert" "alert" "$(k_log_level_name ${KOALEPHANT_LOG_LEVEL_ALERT})"
	assertEquals "Log level name is Crit" "crit" "$(k_log_level_name ${KOALEPHANT_LOG_LEVEL_CRIT})"
	assertEquals "Log level name is Err" "err" "$(k_log_level_name ${KOALEPHANT_LOG_LEVEL_ERR})"
	assertEquals "Log level name is Warning" "warning" "$(k_log_level_name ${KOALEPHANT_LOG_LEVEL_WARNING})"
	assertEquals "Log level name is Notice" "notice" "$(k_log_level_name ${KOALEPHANT_LOG_LEVEL_NOTICE})"
	assertEquals "Log level name is Info" "info" "$(k_log_level_name ${KOALEPHANT_LOG_LEVEL_INFO})"
	assertEquals "Log level name is Debug" "debug" "$(k_log_level_name ${KOALEPHANT_LOG_LEVEL_DEBUG})"

	result="$(k_log_level_name 0 2>/dev/null)"
	assertFalse "Invalid log level name fails"
	assertNull "Invalid Log level name is null" "${result}"
}

test_k_log_level_valid (){
	k_log_level_valid ${KOALEPHANT_LOG_LEVEL_EMERG}
	assertTrue "Log Level Emerg is valid" $?

	k_log_level_valid ${KOALEPHANT_LOG_LEVEL_ALERT}
	assertTrue "Log Level Alert is valid" $?

	k_log_level_valid ${KOALEPHANT_LOG_LEVEL_CRIT}
	assertTrue "Log Level Crit is valid" $?

	k_log_level_valid ${KOALEPHANT_LOG_LEVEL_ERR}
	assertTrue "Log Level Err is valid" $?

	k_log_level_valid ${KOALEPHANT_LOG_LEVEL_WARNING}
	assertTrue "Log Level Warning is valid" $?

	k_log_level_valid ${KOALEPHANT_LOG_LEVEL_NOTICE}
	assertTrue "Log Level Notice is valid" $?

	k_log_level_valid ${KOALEPHANT_LOG_LEVEL_INFO}
	assertTrue "Log Level Info is valid" $?

	k_log_level_valid ${KOALEPHANT_LOG_LEVEL_DEBUG}
	assertTrue "Log Level Debug is valid" $?

	k_log_level_valid 0
	assertFalse "Log Level 0 is invalid" $?

	k_log_level_valid 9
	assertFalse "Log Level 9 is invalid" $?

	k_log_level_valid alert
	assertFalse "Log Level 'alert' is invalid" $?
}

test_k_log_level_parse (){
	# on success, return 0 and output new level to stdout
	result="$(k_log_level_parse ${KOALEPHANT_LOG_LEVEL_EMERG})"
	assertTrue "Log Level Emerg is valid" $?
	assertEquals "Log Level Emerg is output" ${KOALEPHANT_LOG_LEVEL_EMERG} "${result}"

	result="$(k_log_level_parse ${KOALEPHANT_LOG_LEVEL_ALERT})"
	assertTrue "Log Level Alert is valid" $?
	assertEquals "Log Level Alert is output" ${KOALEPHANT_LOG_LEVEL_ALERT} "${result}"

	result="$(k_log_level_parse ${KOALEPHANT_LOG_LEVEL_CRIT})"
	assertTrue "Log Level Crit is valid" $?
	assertEquals "Log Level Crit is output" ${KOALEPHANT_LOG_LEVEL_CRIT} "${result}"

	result="$(k_log_level_parse ${KOALEPHANT_LOG_LEVEL_ERR})"
	assertTrue "Log Level Err is valid" $?
	assertEquals "Log Level Err is output" ${KOALEPHANT_LOG_LEVEL_ERR} "${result}"

	result="$(k_log_level_parse ${KOALEPHANT_LOG_LEVEL_WARNING})"
	assertTrue "Log Level Warning is valid" $?
	assertEquals "Log Level Warning is output" ${KOALEPHANT_LOG_LEVEL_WARNING} "${result}"

	result="$(k_log_level_parse ${KOALEPHANT_LOG_LEVEL_NOTICE})"
	assertTrue "Log Level Notice is valid" $?
	assertEquals "Log Level Notice is output" ${KOALEPHANT_LOG_LEVEL_NOTICE} "${result}"

	result="$(k_log_level_parse ${KOALEPHANT_LOG_LEVEL_INFO})"
	assertTrue "Log Level Info is valid" $?
	assertEquals "Log Level Info is output" ${KOALEPHANT_LOG_LEVEL_INFO} "${result}"

	result="$(k_log_level_parse ${KOALEPHANT_LOG_LEVEL_DEBUG})"
	assertTrue "Log Level Debug is valid" $?
	assertEquals "Log Level Debug is output" ${KOALEPHANT_LOG_LEVEL_DEBUG} "${result}"



	result="$(k_log_level_parse 0 2>&1 1>/dev/null)"
	assertFalse "Log Level 0 is invalid" $?
	assertEquals "Error message is output" 'Invalid log level specified: "0"' "${result}"

	result="$(k_log_level_parse 9 2>&1 1>/dev/null)"
	assertFalse "Log Level 9 is invalid" $?
	assertEquals "Error message is output" 'Invalid log level specified: "9"' "${result}"

	result="$(k_log_level_parse alert 2>&1 1>/dev/null)"
	assertFalse "Log Level 'alert' is invalid" $?
	assertEquals "Error message is output" 'Invalid log level specified: "alert"' "${result}"
}


test_k_option_requires_arg (){
	local output=''

	KOALEPHANT_LOG_SYSLOG=false
	KOALEPHANT_LOG_LEVEL_ACTIVE=${KOALEPHANT_LOG_LEVEL_ERR}

	output="$(k_option_requires_arg --my-opt 2>&1 1>/dev/null)"
	assertFalse 'Unset option value results in error code' $?
	assertEquals 'Unset option value outputs error message' 'Error, --my-opt requires an argument' "${output}"

	output="$(k_option_requires_arg --my-opt foo)"
	assertTrue '"foo" is a valid required option arg value' $?
	assertEquals 'Output is option value with a valid option arg value' foo "${output}"

	output="$(k_option_requires_arg --my-opt -foo 2>&1 1>/dev/null)"
	assertFalse '"-foo" is not a valid required option arg value' $?
	assertEquals 'Invalid option arg value (-foo) outputs error message' 'Error, --my-opt requires an argument' "${output}"

	output="$(k_option_requires_arg --my-opt ' -foo')"
	assertTrue '" -foo" is a valid required option arg value' $?
	assertEquals 'Output is option value with a valid option arg value' ' -foo' "${output}"

	output="$(k_option_requires_arg --my-opt '' 2>&1 1>/dev/null)"
	assertFalse '"" is not a valid required option arg value' $?
	assertEquals 'Invalid option arg value () outputs error message' 'Error, --my-opt requires an argument' "${output}"

	output="$(k_option_requires_arg --my-opt 2>&1 1>/dev/null)"
	assertFalse '(unset) is not a valid required option arg value' $?
	assertEquals 'Invalid option arg value (unset) outputs error message' 'Error, --my-opt requires an argument' "${output}"
}


test_k_option_optional_arg (){
	local output=""

	output="$(k_option_optional_arg '' foo)"
	assertEquals 'Output is option value with a valid option arg value' foo "${output}"

	output="$(k_option_optional_arg '' -foo)"
	assertNull '"-foo" is not a valid optional option arg value' "${output}"

	output="$(k_option_optional_arg '' ' -foo')"
	assertEquals 'Output is option value with a valid option arg value' ' -foo' "${output}"

	output="$(k_option_optional_arg '' '')"
	assertNull  '"" is not a valid optional option arg value' "${output}"

	output="$(k_option_optional_arg)"
	assertNull  '(unset) is not a valid optional option arg value' "${output}"
}

test_k_options_arg_optional (){
	OPTION_VALUE=foo k_options_arg_optional
	assertTrue '"foo" is a valid optional option arg value' $?

	OPTION_VALUE=-foo k_options_arg_optional
	assertFalse '"-foo" is not a valid optional option arg value' $?

	OPTION_VALUE=' -foo' k_options_arg_optional
	assertTrue '" -foo" is a valid optional option arg value' $?

	OPTION_VALUE='' k_options_arg_optional
	assertFalse '"" is not a valid optional option arg value' $?

	unset OPTION_VALUE
	k_options_arg_optional
	assertFalse '(unset) is not a valid optional option arg value' $?
}



test_k_options_arg_required (){
	local output=''
	unset OPTION_NAME

	KOALEPHANT_LOG_SYSLOG=false

	output="$(k_options_arg_required 2>&1 1>/dev/null)"
	assertEquals 'Unset OPTION_NAME results in error code 2' 2 $?
	assertEquals 'Unset OPTION_NAME outputs error message' 'Error, no option name set. Must be used with k_options_split()' "${output}"

	OPTION_NAME='--my-opt'

	output="$(OPTION_VALUE=foo k_options_arg_required 2>&1 1>/dev/null)"
	assertTrue '"foo" is a valid required option arg value' $?
	assertNull 'Output is empty with a valid option arg value' "${output}"

	output="$(OPTION_VALUE=-foo k_options_arg_required 2>&1 1>/dev/null)"
	assertFalse '"-foo" is not a valid required option arg value' $?
	assertEquals 'Invalid option arg value (-foo) outputs error message' 'Error, --my-opt requires an argument' "${output}"

	output="$(OPTION_VALUE=' -foo' k_options_arg_required 2>&1 1>/dev/null)"
	assertTrue '" -foo" is a valid required option arg value' $?
	assertNull 'Output is empty with a valid option arg value' "${output}"

	output="$(OPTION_VALUE='' k_options_arg_required 2>&1 1>/dev/null)"
	assertFalse '"" is not a valid required option arg value' $?
	assertEquals 'Invalid option arg value () outputs error message' 'Error, --my-opt requires an argument' "${output}"

	unset OPTION_VALUE
	output="$(k_options_arg_required 2>&1 1>/dev/null)"
	assertFalse '(unset) is not a valid required option arg value' $?
	assertEquals 'Invalid option arg value (unset) outputs error message' 'Error, --my-opt requires an argument' "${output}"

	unset OPTION_VALUE
	output="$(k_options_arg_required 'You must supply "%s"' 2>&1 1>/dev/null)"
	assertFalse '(unset) is not a valid required option arg value' $?
	assertEquals 'Invalid option arg value (unset) outputs error message' 'You must supply "--my-opt"' "${output}"

}


test_k_options_split (){
	options_split_worker() {
		OPTION_ARG_COUNT=0
		OPTION_NAME=''
		OPTION_VALUE=''

		while k_options_split "$@"; do
			case "$OPTION_NAME" in
				(-a|--arg)
					assertEquals '--arg value is Foo' 'Foo' "${OPTION_VALUE}"
					shift ${OPTION_ARG_COUNT}
				;;

				(-b|--bool)
					if k_options_arg_optional; then
						assertEquals "--bool value is false" 'false' "${OPTION_VALUE}"
						shift ${OPTION_ARG_COUNT}
					else
						shift 1
					fi
				;;

				(--)
					shift 1
					break
				;;

				(*)
					break
				;;
			esac
		done

		if [ $# -gt 0 ]; then
			assertEquals 'Arg1 is HELLO' 'HELLO' "$1"
			shift 1
		fi

		if [ $# -gt 0 ]; then
			assertEquals 'Arg2 is --foo' '--foo' "$1"
			shift 1
		fi

		if [ $# -gt 0 ]; then
			assertEquals 'Arg3 is -BAR' '-BAR' "$1"
			shift 1
		fi
	}


	options_split_worker -a Foo
	options_split_worker -a Foo HELLO
	options_split_worker -a Foo -- HELLO --foo
	options_split_worker -a Foo -- HELLO --foo -BAR
	options_split_worker -aFoo
	options_split_worker -aFoo HELLO
	options_split_worker -aFoo -- HELLO --foo
	options_split_worker -aFoo -- HELLO --foo -BAR
	options_split_worker --arg Foo
	options_split_worker --arg Foo HELLO
	options_split_worker --arg Foo -- HELLO --foo
	options_split_worker --arg Foo -- HELLO --foo -BAR
	options_split_worker --arg=Foo
	options_split_worker --arg=Foo HELLO
	options_split_worker --arg=Foo -- HELLO --foo
	options_split_worker --arg=Foo -- HELLO --foo -BAR
	options_split_worker -b
	options_split_worker -b -- HELLO
	options_split_worker -b -- HELLO --foo
	options_split_worker -b -- HELLO --foo -BAR
	options_split_worker -b false
	options_split_worker -b false HELLO
	options_split_worker -b false -- HELLO --foo
	options_split_worker -b false -- HELLO --foo -BAR
	options_split_worker -bfalse
	options_split_worker -bfalse HELLO
	options_split_worker -bfalse -- HELLO --foo
	options_split_worker -bfalse -- HELLO --foo -BAR
	options_split_worker --bool
	options_split_worker --bool -- HELLO
	options_split_worker --bool -- HELLO --foo
	options_split_worker --bool -- HELLO --foo -BAR
	options_split_worker --bool false
	options_split_worker --bool false HELLO
	options_split_worker --bool false -- HELLO --foo
	options_split_worker --bool false -- HELLO --foo -BAR
	options_split_worker --bool=false
	options_split_worker --bool=false HELLO
	options_split_worker --bool=false -- HELLO --foo
	options_split_worker --bool=false -- HELLO --foo -BAR
	options_split_worker -a Foo -b
	options_split_worker -a Foo -b -- HELLO
	options_split_worker -a Foo -b -- HELLO --foo
	options_split_worker -a Foo -b -- HELLO --foo -BAR
	options_split_worker -a Foo -b false
	options_split_worker -a Foo -b false HELLO
	options_split_worker -a Foo -b false -- HELLO --foo
	options_split_worker -a Foo -b false -- HELLO --foo -BAR
	options_split_worker -a Foo -bfalse
	options_split_worker -a Foo -bfalse HELLO
	options_split_worker -a Foo -bfalse -- HELLO --foo
	options_split_worker -a Foo -bfalse -- HELLO --foo -BAR
	options_split_worker -a Foo --bool
	options_split_worker -a Foo --bool -- HELLO
	options_split_worker -a Foo --bool -- HELLO --foo
	options_split_worker -a Foo --bool -- HELLO --foo -BAR
	options_split_worker -a Foo --bool false
	options_split_worker -a Foo --bool false HELLO
	options_split_worker -a Foo --bool false -- HELLO --foo
	options_split_worker -a Foo --bool false -- HELLO --foo -BAR
	options_split_worker -a Foo --bool=false
	options_split_worker -a Foo --bool=false HELLO
	options_split_worker -a Foo --bool=false -- HELLO --foo
	options_split_worker -a Foo --bool=false -- HELLO --foo -BAR
	options_split_worker -aFoo -b
	options_split_worker -aFoo -b -- HELLO
	options_split_worker -aFoo -b -- HELLO --foo
	options_split_worker -aFoo -b -- HELLO --foo -BAR
	options_split_worker -aFoo -b false
	options_split_worker -aFoo -b false HELLO
	options_split_worker -aFoo -b false -- HELLO --foo
	options_split_worker -aFoo -b false -- HELLO --foo -BAR
	options_split_worker -aFoo -bfalse
	options_split_worker -aFoo -bfalse HELLO
	options_split_worker -aFoo -bfalse -- HELLO --foo
	options_split_worker -aFoo -bfalse -- HELLO --foo -BAR
	options_split_worker -aFoo --bool
	options_split_worker -aFoo --bool -- HELLO
	options_split_worker -aFoo --bool -- HELLO --foo
	options_split_worker -aFoo --bool -- HELLO --foo -BAR
	options_split_worker -aFoo --bool false
	options_split_worker -aFoo --bool false HELLO
	options_split_worker -aFoo --bool false -- HELLO --foo
	options_split_worker -aFoo --bool false -- HELLO --foo -BAR
	options_split_worker -aFoo --bool=false
	options_split_worker -aFoo --bool=false HELLO
	options_split_worker -aFoo --bool=false -- HELLO --foo
	options_split_worker -aFoo --bool=false -- HELLO --foo -BAR
	options_split_worker --arg Foo -b
	options_split_worker --arg Foo -b -- HELLO
	options_split_worker --arg Foo -b -- HELLO --foo
	options_split_worker --arg Foo -b -- HELLO --foo -BAR
	options_split_worker --arg Foo -b false
	options_split_worker --arg Foo -b false HELLO
	options_split_worker --arg Foo -b false -- HELLO --foo
	options_split_worker --arg Foo -b false -- HELLO --foo -BAR
	options_split_worker --arg Foo -bfalse
	options_split_worker --arg Foo -bfalse HELLO
	options_split_worker --arg Foo -bfalse -- HELLO --foo
	options_split_worker --arg Foo -bfalse -- HELLO --foo -BAR
	options_split_worker --arg Foo --bool
	options_split_worker --arg Foo --bool -- HELLO
	options_split_worker --arg Foo --bool -- HELLO --foo
	options_split_worker --arg Foo --bool -- HELLO --foo -BAR
	options_split_worker --arg Foo --bool false
	options_split_worker --arg Foo --bool false HELLO
	options_split_worker --arg Foo --bool false -- HELLO --foo
	options_split_worker --arg Foo --bool false -- HELLO --foo -BAR
	options_split_worker --arg Foo --bool=false
	options_split_worker --arg Foo --bool=false HELLO
	options_split_worker --arg Foo --bool=false -- HELLO --foo
	options_split_worker --arg Foo --bool=false -- HELLO --foo -BAR
	options_split_worker --arg=Foo -b
	options_split_worker --arg=Foo -b -- HELLO
	options_split_worker --arg=Foo -b -- HELLO --foo
	options_split_worker --arg=Foo -b -- HELLO --foo -BAR
	options_split_worker --arg=Foo -b false
	options_split_worker --arg=Foo -b false HELLO
	options_split_worker --arg=Foo -b false -- HELLO --foo
	options_split_worker --arg=Foo -b false -- HELLO --foo -BAR
	options_split_worker --arg=Foo -bfalse
	options_split_worker --arg=Foo -bfalse HELLO
	options_split_worker --arg=Foo -bfalse -- HELLO --foo
	options_split_worker --arg=Foo -bfalse -- HELLO --foo -BAR
	options_split_worker --arg=Foo --bool
	options_split_worker --arg=Foo --bool -- HELLO
	options_split_worker --arg=Foo --bool -- HELLO --foo
	options_split_worker --arg=Foo --bool -- HELLO --foo -BAR
	options_split_worker --arg=Foo --bool false
	options_split_worker --arg=Foo --bool false HELLO
	options_split_worker --arg=Foo --bool false -- HELLO --foo
	options_split_worker --arg=Foo --bool false -- HELLO --foo -BAR
	options_split_worker --arg=Foo --bool=false
	options_split_worker --arg=Foo --bool=false HELLO
	options_split_worker --arg=Foo --bool=false -- HELLO --foo
	options_split_worker --arg=Foo --bool=false -- HELLO --foo -BAR
}



test_k_require_root (){
	not_root() {
		local output
		id () {
			printf -- '%d\n' 1
		}

		output="$(k_require_root 2>&1 1>/dev/null)"
		assertFalse 'UID other than 0 fails' $?
		assertEquals 'Output is an error message' "$(k_tool_name) must be run as root" "${output}"

		output="$(k_require_root 'Foo is a Bar' 2>&1 1>/dev/null)"
		assertEquals 'Output is a custom + error message' "$(printf -- '%s\n' 'Foo is a Bar' "$(k_tool_name) must be run as root")" "${output}"
	}

	is_root() {
		id () {
			printf -- '%d\n' 0
		}

		k_require_root
		assertTrue 'UID 0 passes' $?
	}
	is_root
	not_root
}



test_k_requires_args (){
	local output tmplate='%s requires at least %d arguments, %d given\n'

	output="$(k_requires_args 2>&1 1>/dev/null)"
	assertFalse 'Return is error without arguments' $?
	assertEquals 'Output is an error message' "$(printf -- "${tmplate}" k_requires_args 2 0)" "${output}"

	output="$(k_requires_args foo_func 2>&1 1>/dev/null)"
	assertFalse 'Return is error with 1 argument' $?
	assertEquals 'Output is an error message' "$(printf -- "${tmplate}" k_requires_args 2 1)" "${output}"

	output="$(k_requires_args foo_func 0 2>&1 1>/dev/null)"
	assertFalse 'Return is error with 2 arguments if second is < 1' $?
	assertEquals 'Output is an error message with 2 argument if second is < 1' "$(printf -- "${tmplate}" foo_func 1 0)" "${output}"

	output="$(k_requires_args foo_func 1 2 2>&1 1>/dev/null)"
	assertFalse 'Return is error with 3 arguments if second is < third' $?
	assertEquals 'Output is an error message with 2 argument if second is < third' "$(printf -- "${tmplate}" foo_func 2 1)" "${output}"

	output="$(k_requires_args foo_func 1 2>&1 1>/dev/null)"
	assertTrue 'Return is success with 2 arguments if second is >= 1' $?

	output="$(k_requires_args foo_func 2 2  2>&1 1>/dev/null)"
	assertTrue 'Return is success with 3 arguments if second is >= third' $?

	output="$(k_requires_args foo_func 3 2  2>&1 1>/dev/null)"
	assertTrue 'Return is success with 3 arguments if second is >= third' $?
}



test_k_tool_usage (){
	assertEquals 'Output is correct format' "$(printf -- 'Usage: %s%s%s\n' "$(k_tool_name)" "$(k_tool_options_arguments)" "$(k_tool_arguments)")" "$(k_tool_usage)"
}

test_k_usage (){
	assertEquals 'Output is correct format' "$(printf -- '%s\n%s\n\n%s\n\n%s\n' "$(k_tool_usage)" "$(k_tool_description)" "$(k_tool_options)" "$(k_tool_environment)")" "$(k_usage)"
}

test_k_version_helper (){
	local output

	k_version_helper false false false
	assertFalse 'Return is error when first arg is false' $?

	output="$(k_version_helper true false false)"
	assertTrue 'Return is success when first arg is true' $?
	assertEquals 'Output is full tool version info when args 2 and 3 are false' "$(k_version)" "${output}"

	output="$(k_version_helper true false true )"
	assertTrue 'Return is success when first and last args are true' $?
	assertEquals 'Output is full tool + lib version info when args are true false true' "$(k_version; k_library_version)" "${output}"

	output="$(k_version_helper true true false)"
	assertTrue 'Return is success when first two args are true' $?
	assertEquals 'Output is basic tool version info when args are true true false' "$(k_tool_version)" "${output}"

	output="$(k_version_helper true true true)"
	assertTrue 'Return is success when all args are true' $?
	assertEquals 'Output is basic tool + lib version info when args are true true true' "$(k_tool_version; printf -- '%s\n' "${KOALEPHANT_LIB_VERSION}")" "${output}"
}





